from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from .GeometryShellSection import GeometryShellSection
from .RebarLayers import RebarLayers
from .TransverseShearShell import TransverseShearShell
from ..UtilityAndView.SymbolicConstant import abaqusConstants as C
from ..UtilityAndView.abaqusConstants import BENDING, Boolean, DEFAULT, GRADIENT, NO_IDEALIZATION, OFF, SIMPSON, UNIFORM


@abaqus_class_doc
class HomogeneousShellSection(GeometryShellSection):
    """The HomogeneousShellSection object defines the properties of a shell section.
    The HomogeneousShellSection object is derived from the GeometryShellSection object.

    .. note::
        This object can be accessed by::

            import section
            mdb.models[name].parts[name].compositeLayups[i].section
            mdb.models[name].sections[name]
            import odbSection
            session.odbs[name].sections[name]

        The corresponding analysis keywords are:

        - SHELL SECTION
        - SHELL GENERAL SECTION
    """

    #: A :py:class:`~abaqus.Section.RebarLayers.RebarLayers` object specifying reinforcement properties.
    rebarLayers: Optional[RebarLayers] = None

    #: A :py:class:`~abaqus.Section.TransverseShearShell.TransverseShearShell` object specifying the transverse shear stiffness properties.
    transverseShear: Optional[TransverseShearShell] = None

    #: A String specifying the repository key.
    name: str

    #: A String specifying the name of the section material.
    material: str

    #: A Float specifying the thickness of the section. The **thickness** argument applies only
    #: when **thicknessType** = UNIFORM. The default value is 0.0.
    thickness: float = 0

    #: An Int specifying the number of integration points to be used through the section.
    #: Possible values are **numIntPts** >> 0. The default value is 5.To use the default settings
    #: of the analysis products, set **numIntPts** to 5 if **integrationRule** = SIMPSON or set
    #: **numIntPts** to 7 if **integrationRule** = GAUSS.
    numIntPts: int = 5

    #: A SymbolicConstant specifying the distribution used for defining the thickness of the
    #: elements. Possible values are UNIFORM, ANALYTICAL_FIELD, DISCRETE_FIELD,
    #: NODAL_ANALYTICAL_FIELD, and NODAL_DISCRETE_FIELD. The default value is UNIFORM.
    thicknessType: Literal[
        C.UNIFORM,
        C.ANALYTICAL_FIELD,
        C.DISCRETE_FIELD,
        C.NODAL_ANALYTICAL_FIELD,
        C.NODAL_DISCRETE_FIELD,
    ] = UNIFORM

    #: A Boolean specifying whether the shell section properties are specified by the user
    #: prior to the analysis (ON) or integrated during the analysis (OFF). The default value is
    #: OFF.
    preIntegrate: Boolean = OFF

    #: A SymbolicConstant specifying whether to use the default value for the Poisson's ratio.
    #: Possible values are:DEFAULT, specifying that the default value for the Poisson's ratio
    #: is 0.5 in an Abaqus/Standard analysis and is obtained from the material definition in an
    #: Abaqus/Explicit analysis.VALUE, specifying that the Poisson's ratio used in the analysis
    #: is the value provided in **poisson**.The default value is DEFAULT.
    poissonDefinition: Literal[C.VALUE, C.DEFAULT] = DEFAULT

    #: A Float specifying the Poisson's ratio. Possible values are −1.0 ≤ **poisson** ≤ 0.5.
    #: This argument is valid only when **poissonDefinition** = VALUE. The default value is 0.5.
    poisson: float = 0

    #: A SymbolicConstant specifying the shell section integration rule. Possible values are
    #: SIMPSON and GAUSS. The default value is SIMPSON.
    integrationRule: Literal[C.SIMPSON, C.GAUSS] = SIMPSON

    #: A SymbolicConstant specifying the mode used for temperature and field variable input
    #: across the section thickness. Possible values are GRADIENT and POINTWISE. The default
    #: value is GRADIENT.
    temperature: Literal[C.GRADIENT, C.POINTWISE] = GRADIENT

    #: A SymbolicConstant specifying the mechanical idealization used for the section
    #: calculations. This member is only applicable when **preIntegrate** is set to ON. Possible
    #: values are NO_IDEALIZATION, SMEAR_ALL_LAYERS, MEMBRANE, and BENDING. The default value
    #: is NO_IDEALIZATION.
    idealization: Literal[C.NO_IDEALIZATION, C.SMEAR_ALL_LAYERS, C.MEMBRANE, C.BENDING] = NO_IDEALIZATION

    #: None or an Int specifying the number of temperature points to be input. This argument is
    #: valid only when **temperature** = POINTWISE. The default value is None.
    nTemp: Optional[int] = None

    #: None or a Float specifying the effective thickness modulus. This argument is relevant
    #: only for continuum shells and must be used in conjunction with the argument **poisson**.
    #: The default value is None.
    thicknessModulus: Optional[float] = None

    #: A Boolean specifying whether or not to use the value of **density**. The default value is
    #: OFF.
    useDensity: Boolean = OFF

    #: A Float specifying the value of density to apply to this section. The default value is
    #: 0.0.
    density: float = 0

    #: A String specifying the name of the AnalyticalField or DiscreteField object used to
    #: define the thickness of the shell elements. The **thicknessField** argument applies only
    #: when **thicknessType** = ANALYTICAL_FIELD or **thicknessType** = DISCRETE_FIELD. The default
    #: value is an empty string.
    thicknessField: str = ""

    #: A String specifying the name of the AnalyticalField or DiscreteField object used to
    #: define the thickness of the shell elements at each node. The **nodalThicknessField**
    #: argument applies only when **thicknessType** = NODAL_ANALYTICAL_FIELD or
    #: **thicknessType** = NODAL_DISCRETE_FIELD. The default value is an empty string.
    nodalThicknessField: str = ""

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        material: str,
        thickness: float = 0,
        numIntPts: int = 5,
        thicknessType: Literal[
            C.UNIFORM,
            C.ANALYTICAL_FIELD,
            C.DISCRETE_FIELD,
            C.NODAL_ANALYTICAL_FIELD,
            C.NODAL_DISCRETE_FIELD,
        ] = UNIFORM,
        preIntegrate: Boolean = OFF,
        poissonDefinition: Literal[C.VALUE, C.DEFAULT] = DEFAULT,
        poisson: float = 0,
        integrationRule: Literal[C.SIMPSON, C.GAUSS] = SIMPSON,
        temperature: Literal[C.GRADIENT, C.POINTWISE] = GRADIENT,
        idealization: Literal[C.NO_IDEALIZATION, C.SMEAR_ALL_LAYERS, C.MEMBRANE, BENDING] = NO_IDEALIZATION,
        nTemp: Optional[int] = None,
        thicknessModulus: Optional[float] = None,
        useDensity: Boolean = OFF,
        density: float = 0,
        thicknessField: str = "",
        nodalThicknessField: str = "",
    ) -> None:
        """This method creates a HomogeneousShellSection object.

        .. note::
            This function can be accessed by::

                mdb.models[name].parts[name].compositeLayups[i].HomogeneousShellSection
                mdb.models[name].HomogeneousShellSection
                session.odbs[name].HomogeneousShellSection

        Parameters
        ----------
        name
            A String specifying the repository key.
        material
            A String specifying the name of the section material.
        thickness
            A Float specifying the thickness of the section. The **thickness** argument applies only
            when **thicknessType** = UNIFORM. The default value is 0.0.
        numIntPts
            An Int specifying the number of integration points to be used through the section.
            Possible values are **numIntPts** >> 0. The default value is 5.To use the default settings
            of the analysis products, set **numIntPts** to 5 if **integrationRule** = SIMPSON or set
            **numIntPts** to 7 if **integrationRule** = GAUSS.
        thicknessType
            A SymbolicConstant specifying the distribution used for defining the thickness of the
            elements. Possible values are UNIFORM, ANALYTICAL_FIELD, DISCRETE_FIELD,
            NODAL_ANALYTICAL_FIELD, and NODAL_DISCRETE_FIELD. The default value is UNIFORM.
        preIntegrate
            A Boolean specifying whether the shell section properties are specified by the user
            prior to the analysis (ON) or integrated during the analysis (OFF). The default value is
            OFF.
        poissonDefinition
            A SymbolicConstant specifying whether to use the default value for the Poisson's ratio.
            Possible values are:DEFAULT, specifying that the default value for the Poisson's ratio
            is 0.5 in an Abaqus/Standard analysis and is obtained from the material definition in an
            Abaqus/Explicit analysis.VALUE, specifying that the Poisson's ratio used in the analysis
            is the value provided in **poisson**.The default value is DEFAULT.
        poisson
            A Float specifying the Poisson's ratio. Possible values are −1.0 ≤ **poisson** ≤ 0.5.
            This argument is valid only when **poissonDefinition** = VALUE. The default value is 0.5.
        integrationRule
            A SymbolicConstant specifying the shell section integration rule. Possible values are
            SIMPSON and GAUSS. The default value is SIMPSON.
        temperature
            A SymbolicConstant specifying the mode used for temperature and field variable input
            across the section thickness. Possible values are GRADIENT and POINTWISE. The default
            value is GRADIENT.
        idealization
            A SymbolicConstant specifying the mechanical idealization used for the section
            calculations. This member is only applicable when **preIntegrate** is set to ON. Possible
            values are NO_IDEALIZATION, SMEAR_ALL_LAYERS, MEMBRANE, and BENDING. The default value
            is NO_IDEALIZATION.
        nTemp
            None or an Int specifying the number of temperature points to be input. This argument is
            valid only when **temperature** = POINTWISE. The default value is None.
        thicknessModulus
            None or a Float specifying the effective thickness modulus. This argument is relevant
            only for continuum shells and must be used in conjunction with the argument **poisson**.
            The default value is None.
        useDensity
            A Boolean specifying whether or not to use the value of **density**. The default value is
            OFF.
        density
            A Float specifying the value of density to apply to this section. The default value is
            0.0.
        thicknessField
            A String specifying the name of the AnalyticalField or DiscreteField object used to
            define the thickness of the shell elements. The **thicknessField** argument applies only
            when **thicknessType** = ANALYTICAL_FIELD or **thicknessType** = DISCRETE_FIELD. The default
            value is an empty string.
        nodalThicknessField
            A String specifying the name of the AnalyticalField or DiscreteField object used to
            define the thickness of the shell elements at each node. The **nodalThicknessField**
            argument applies only when **thicknessType** = NODAL_ANALYTICAL_FIELD or
            **thicknessType** = NODAL_DISCRETE_FIELD. The default value is an empty string.

        Returns
        -------
        HomogeneousShellSection
            A :py:class:`~abaqus.Section.HomogeneousShellSection.HomogeneousShellSection` object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        thickness: float = 0,
        numIntPts: int = 5,
        thicknessType: Literal[
            C.UNIFORM,
            C.ANALYTICAL_FIELD,
            C.DISCRETE_FIELD,
            C.NODAL_ANALYTICAL_FIELD,
            C.NODAL_DISCRETE_FIELD,
        ] = UNIFORM,
        preIntegrate: Boolean = OFF,
        poissonDefinition: Literal[C.VALUE, C.DEFAULT] = DEFAULT,
        poisson: float = 0,
        integrationRule: Literal[C.SIMPSON, C.GAUSS] = SIMPSON,
        temperature: Literal[C.GRADIENT, C.POINTWISE] = GRADIENT,
        idealization: Literal[C.NO_IDEALIZATION, C.SMEAR_ALL_LAYERS, C.MEMBRANE, BENDING] = NO_IDEALIZATION,
        nTemp: Optional[int] = None,
        thicknessModulus: Optional[float] = None,
        useDensity: Boolean = OFF,
        density: float = 0,
        thicknessField: str = "",
        nodalThicknessField: str = "",
    ) -> None:
        """This method modifies the HomogeneousShellSection object.

        Parameters
        ----------
        thickness
            A Float specifying the thickness of the section. The **thickness** argument applies only
            when **thicknessType** = UNIFORM. The default value is 0.0.
        numIntPts
            An Int specifying the number of integration points to be used through the section.
            Possible values are **numIntPts** >> 0. The default value is 5.To use the default settings
            of the analysis products, set **numIntPts** to 5 if **integrationRule** = SIMPSON or set
            **numIntPts** to 7 if **integrationRule** = GAUSS.
        thicknessType
            A SymbolicConstant specifying the distribution used for defining the thickness of the
            elements. Possible values are UNIFORM, ANALYTICAL_FIELD, DISCRETE_FIELD,
            NODAL_ANALYTICAL_FIELD, and NODAL_DISCRETE_FIELD. The default value is UNIFORM.
        preIntegrate
            A Boolean specifying whether the shell section properties are specified by the user
            prior to the analysis (ON) or integrated during the analysis (OFF). The default value is
            OFF.
        poissonDefinition
            A SymbolicConstant specifying whether to use the default value for the Poisson's ratio.
            Possible values are:DEFAULT, specifying that the default value for the Poisson's ratio
            is 0.5 in an Abaqus/Standard analysis and is obtained from the material definition in an
            Abaqus/Explicit analysis.VALUE, specifying that the Poisson's ratio used in the analysis
            is the value provided in **poisson**.The default value is DEFAULT.
        poisson
            A Float specifying the Poisson's ratio. Possible values are −1.0 ≤ **poisson** ≤ 0.5.
            This argument is valid only when **poissonDefinition** = VALUE. The default value is 0.5.
        integrationRule
            A SymbolicConstant specifying the shell section integration rule. Possible values are
            SIMPSON and GAUSS. The default value is SIMPSON.
        temperature
            A SymbolicConstant specifying the mode used for temperature and field variable input
            across the section thickness. Possible values are GRADIENT and POINTWISE. The default
            value is GRADIENT.
        idealization
            A SymbolicConstant specifying the mechanical idealization used for the section
            calculations. This member is only applicable when **preIntegrate** is set to ON. Possible
            values are NO_IDEALIZATION, SMEAR_ALL_LAYERS, MEMBRANE, and BENDING. The default value
            is NO_IDEALIZATION.
        nTemp
            None or an Int specifying the number of temperature points to be input. This argument is
            valid only when **temperature** = POINTWISE. The default value is None.
        thicknessModulus
            None or a Float specifying the effective thickness modulus. This argument is relevant
            only for continuum shells and must be used in conjunction with the argument **poisson**.
            The default value is None.
        useDensity
            A Boolean specifying whether or not to use the value of **density**. The default value is
            OFF.
        density
            A Float specifying the value of density to apply to this section. The default value is
            0.0.
        thicknessField
            A String specifying the name of the AnalyticalField or DiscreteField object used to
            define the thickness of the shell elements. The **thicknessField** argument applies only
            when **thicknessType** = ANALYTICAL_FIELD or **thicknessType** = DISCRETE_FIELD. The default
            value is an empty string.
        nodalThicknessField
            A String specifying the name of the AnalyticalField or DiscreteField object used to
            define the thickness of the shell elements at each node. The **nodalThicknessField**
            argument applies only when **thicknessType** = NODAL_ANALYTICAL_FIELD or
            **thicknessType** = NODAL_DISCRETE_FIELD. The default value is an empty string.
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from ..UtilityAndView.abaqusConstants import Boolean, FILLED_CIRCLE, ON, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class SymbolStyle:
    """The SymbolStyle object is used to define the marker properties to be used when drawing
    curves.
    SymbolStyle objects can be created using the methods described below.

    .. note::
        This object can be accessed by::

            import visualization
            session.charts[name].axes1[i].axisData.curves[i].symbolStyle
            session.charts[name].axes2[i].axisData.curves[i].symbolStyle
            session.charts[name].curves[name].symbolStyle
            session.curves[name].symbolStyle
            session.defaultChartOptions.defaultAxis1Options.axisData.curves[i].symbolStyle
            session.defaultChartOptions.defaultAxis2Options.axisData.curves[i].symbolStyle
            session.xyPlots[name].charts[name].axes1[i].axisData.curves[i].symbolStyle
            session.xyPlots[name].charts[name].axes2[i].axisData.curves[i].symbolStyle
            session.xyPlots[name].charts[name].curves[name].symbolStyle
            session.xyPlots[name].curves[name].symbolStyle
    """

    #: A String specifying the color to be used when drawing a marker with this SymbolStyle
    #: object. The default value is "White".
    color: str = ""

    #: A Boolean specifying whether to draw the marker when using this SymbolStyle object. The
    #: default value is ON.
    show: Boolean = ON

    #: A SymbolicConstant specifying the marker type be used when drawing symbols using this
    #: SymbolStyle object. Possible values are:

    #: - FILLED_CIRCLE
    #: - FILLED_SQUARE
    #: - FILLED_DIAMOND
    #: - FILLED_TRI
    #: - HOLLOW_CIRCLE
    #: - HOLLOW_SQUARE
    #: - HOLLOW_DIAMOND
    #: - HOLLOW_TRI
    #: - CROSS
    #: - XMARKER
    #: - POINT
    #:
    #: The default value is FILLED_CIRCLE.
    marker: SymbolicConstant = FILLED_CIRCLE

    #: A Float specifying the marker size to be used when drawing markers using this
    #: SymbolStyle object. The default value is 2.0.
    size: float = 2

    @abaqus_method_doc
    def __init__(
        self,
        color: str = "",
        show: Boolean = ON,
        marker: Literal[
            C.CROSS,
            C.FILLED_DIAMOND,
            C.FILLED_SQUARE,
            C.POINT,
            C.HOLLOW_CIRCLE,
            C.HOLLOW_TRI,
            C.HOLLOW_SQUARE,
            C.FILLED_TRI,
            C.HOLLOW_DIAMOND,
            C.FILLED_CIRCLE,
            C.XMARKER,
        ] = FILLED_CIRCLE,
        size: float = 2,
    ):
        """This method creates a SymbolStyle object.

        .. note::
            This function can be accessed by::

                session.SymbolStyle
                xyPlot.SymbolStyle

        Parameters
        ----------
        color
            A String specifying the color to be used when drawing a marker with this SymbolStyle
            object. The default value is "White".
        show
            A Boolean specifying whether to draw the marker when using this SymbolStyle object. The
            default value is ON.
        marker
            A SymbolicConstant specifying the marker type be used when drawing symbols using this
            SymbolStyle object. Possible values are:

            - FILLED_CIRCLE
            - FILLED_SQUARE
            - FILLED_DIAMOND
            - FILLED_TRI
            - HOLLOW_CIRCLE
            - HOLLOW_SQUARE
            - HOLLOW_DIAMOND
            - HOLLOW_TRI
            - CROSS
            - XMARKER
            - POINT

            The default value is FILLED_CIRCLE.
        size
            A Float specifying the marker size to be used when drawing markers using this
            SymbolStyle object. The default value is 2.0.

        Returns
        -------
        SymbolStyle
            A :py:class:`~abaqus.XY.SymbolStyle.SymbolStyle` object.

        Raises
        ------
        ColorError
        """
        ...

    @abaqus_method_doc
    def setValues(
        self,
        color: str = "",
        show: Boolean = ON,
        marker: Literal[
            C.CROSS,
            C.FILLED_DIAMOND,
            C.FILLED_SQUARE,
            C.POINT,
            C.HOLLOW_CIRCLE,
            C.HOLLOW_TRI,
            C.HOLLOW_SQUARE,
            C.FILLED_TRI,
            C.HOLLOW_DIAMOND,
            C.FILLED_CIRCLE,
            C.XMARKER,
        ] = FILLED_CIRCLE,
        size: float = 2,
    ):
        """This method modifies the SymbolStyle object.

        Parameters
        ----------
        color
            A String specifying the color to be used when drawing a marker with this SymbolStyle
            object. The default value is "White".
        show
            A Boolean specifying whether to draw the marker when using this SymbolStyle object. The
            default value is ON.
        marker
            A SymbolicConstant specifying the marker type be used when drawing symbols using this
            SymbolStyle object. Possible values are:

            - FILLED_CIRCLE
            - FILLED_SQUARE
            - FILLED_DIAMOND
            - FILLED_TRI
            - HOLLOW_CIRCLE
            - HOLLOW_SQUARE
            - HOLLOW_DIAMOND
            - HOLLOW_TRI
            - CROSS
            - XMARKER
            - POINT

            The default value is FILLED_CIRCLE.
        size
            A Float specifying the marker size to be used when drawing markers using this
            SymbolStyle object. The default value is 2.0.
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from .LoadState import LoadState
from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class LineLoadState(LoadState):
    """The LineLoadState object stores the propagating data of a line load in a step. One
    instance of this object is created internally by the LineLoad object for each step. The
    instance is also deleted internally by the LineLoad object.
    The LineLoadState object has no constructor or methods.
    The LineLoadState object is derived from the LoadState object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].steps[name].loadStates[name]

        The corresponding analysis keywords are:

        - DLOAD
    """

    #: A Float or a Complex specifying the load component in the global or the beam local
    #: 1-direction.
    comp1: Optional[float] = None

    #: A Float or a Complex specifying the load component in the global or the beam local
    #: 2-direction.
    comp2: Optional[float] = None

    #: A Float or a Complex specifying the load component in the global 3-direction.
    comp3: Optional[float] = None

    #: A SymbolicConstant specifying the propagation state of the load component in the global
    #: or the beam local 1-direction. Possible values are UNSET, SET, UNCHANGED, and FREED.
    comp1State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the load component in the global
    #: or the beam local 2-direction. Possible values are UNSET, SET, UNCHANGED, and FREED.
    comp2State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the load component in the global
    #: 3-direction. Possible values are UNSET, SET, UNCHANGED, and FREED.
    comp3State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the **amplitude** member. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    amplitudeState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the LoadState object. Possible
    #: values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: Optional[SymbolicConstant] = None

    #: A String specifying the name of the amplitude reference. The String is empty if the load
    #: has no amplitude reference.
    amplitude: str = ""



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ...UtilityAndView.abaqusConstants import Boolean, OFF


@abaqus_class_doc
class UniaxialTestData:
    r"""The UniaxialTestData object provides uniaxial test data (compression and/or tension).

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].hyperelastic.uniaxialTestData
            mdb.models[name].materials[name].hyperfoam.uniaxialTestData
            mdb.models[name].materials[name].lowDensityFoam.uniaxialCompressionTestData
            mdb.models[name].materials[name].lowDensityFoam.uniaxialTensionTestData
            mdb.models[name].materials[name].mullinsEffect.uniaxialTests[i]
            import odbMaterial
            session.odbs[name].materials[name].hyperelastic.uniaxialTestData
            session.odbs[name].materials[name].hyperfoam.uniaxialTestData
            session.odbs[name].materials[name].lowDensityFoam.uniaxialCompressionTestData
            session.odbs[name].materials[name].lowDensityFoam.uniaxialTensionTestData
            session.odbs[name].materials[name].mullinsEffect.uniaxialTests[i]

        The table data for this object are:

        - For a hyperelastic material model, the table data specify the following:

            - Nominal stress, :math:`T_{U}`.
            - Nominal strain, :math:`\epsilon_{U}`.
        - For a hyperfoam material model, the table data specify the following:

            - Nominal stress, :math:`T_{L}`.
            - Nominal strain, :math:`\epsilon_{U}`.
            - Nominal lateral strain, :math:`\epsilon_{2}=\epsilon_{3}`. The default value is 0 .
        - For a low-density foam material model, the table data specify the following:

            - Nominal stress, :math:`T_{U}`.
            - Nominal strain, :math:`\epsilon_{U}`.
            - Nominal strain rate, :math:`\dot{\epsilon_{U}}`.

        The corresponding analysis keywords are:

        - UNIAXIAL TEST DATA
    """

    @abaqus_method_doc
    def __init__(
        self,
        table: tuple,
        smoothing: Optional[int] = None,
        lateralNominalStrain: Boolean = OFF,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
    ):
        """This method creates a UniaxialTestData object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].hyperelastic.UniaxialTestData
                mdb.models[name].materials[name].hyperfoam.UniaxialTestData
                mdb.models[name].materials[name].lowDensityFoam.UniaxialTestData
                mdb.models[name].materials[name].mullinsEffect.UniaxialTestData
                session.odbs[name].materials[name].hyperelastic.UniaxialTestData
                session.odbs[name].materials[name].hyperfoam.UniaxialTestData
                session.odbs[name].materials[name].lowDensityFoam.UniaxialTestData
                session.odbs[name].materials[name].mullinsEffect.UniaxialTestData

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.
        smoothing
            None or an Int specifying the value for smoothing. If **smoothing** = None, no smoothing is
            employed. The default value is None.
        lateralNominalStrain
            A Boolean specifying whether to include lateral nominal strain. The default value is
            OFF.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.

        Returns
        -------
        UniaxialTestData
            A :py:class:`~abaqus.Material.TestData.UniaxialTestData.UniaxialTestData` object.
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the UniaxialTestData object."""
        ...



from __future__ import annotations

from typing import Optional, Tuple, TYPE_CHECKING

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .MeshFace import MeshFace
from ..Datum.DatumCsys import DatumCsys

if TYPE_CHECKING:  # to avoid circular imports
    from .MeshEdge import MeshEdge
    from .MeshElement import MeshElement
    from .MeshNodeArray import MeshNodeArray


@abaqus_class_doc
class MeshNode:
    """The MeshNode object refers to a node of a native mesh or an orphan mesh. A MeshNode
    object can be accessed via a part or part instance using an index that refers to the
    internal numbering of the node repository. The index does not refer to the node label.

    .. note::
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].allInternalSets[name].nodes[i]
            mdb.models[name].parts[name].allInternalSurfaces[name].nodes[i]
            mdb.models[name].parts[name].allSets[name].nodes[i]
            mdb.models[name].parts[name].allSurfaces[name].nodes[i]
            mdb.models[name].parts[name].nodes[i]
            mdb.models[name].parts[name].retainedNodes[i]
            mdb.models[name].parts[name].sets[name].nodes[i]
            mdb.models[name].parts[name].surfaces[name].nodes[i]
            import assembly
            mdb.models[name].rootAssembly.allInstances[name].nodes[i]
            mdb.models[name].rootAssembly.allInstances[name].sets[name].nodes[i]
            mdb.models[name].rootAssembly.allInstances[name].surfaces[name].nodes[i]
            mdb.models[name].rootAssembly.allInternalSets[name].nodes[i]
            mdb.models[name].rootAssembly.allInternalSurfaces[name].nodes[i]
            mdb.models[name].rootAssembly.allSets[name].nodes[i]
            mdb.models[name].rootAssembly.allSurfaces[name].nodes[i]
            mdb.models[name].rootAssembly.instances[name].nodes[i]
            mdb.models[name].rootAssembly.instances[name].sets[name].nodes[i]
            mdb.models[name].rootAssembly.instances[name].surfaces[name].nodes[i]
            mdb.models[name].rootAssembly.modelInstances[i].nodes[i]
            mdb.models[name].rootAssembly.modelInstances[i].sets[name].nodes[i]
            mdb.models[name].rootAssembly.modelInstances[i].surfaces[name].nodes[i]
            mdb.models[name].rootAssembly.nodes[i]
            mdb.models[name].rootAssembly.sets[name].nodes[i]
            mdb.models[name].rootAssembly.surfaces[name].nodes[i]
    """

    #: An Int specifying the node label.
    label: Optional[int] = None

    #: A String specifying the name of the part instance that owns this node.
    instanceName: str = ""

    #: A tuple of three Floats specifying the coordinates of the new node.
    coordinates: Tuple[float, float, float]

    @abaqus_method_doc
    def __init__(
        self,
        coordinates: Tuple[float, float, float],
        localCsys: Optional[DatumCsys] = None,
        label: Optional[int] = None,
    ) -> None:
        """This method creates a node on an orphan mesh part.

        .. note::
            This function can be accessed by::

                mdb.models[name].parts[name].Node

        Parameters
        ----------
        coordinates
            A sequence of three Floats specifying the coordinates of the new node.
        localCsys
            A :py:class:`~abaqus.Datum.DatumCsys.DatumCsys` object specifying the local coordinate system. If unspecified, the global
            coordinate system will be used.
        label
            An Int specifying the node label.

        Returns
        -------
        node: MeshNode
            A :py:class:`~abaqus.Mesh.MeshNode.MeshNode` object
        """
        ...

    @abaqus_method_doc
    def getElemEdges(self) -> Tuple[MeshEdge, ...]:
        """This method returns a tuple of element edge objects that share the node.

        Returns
        -------
        edges: Sequence[MeshEdge]
            A tuple of MeshEdge objects
        """
        ...

    @abaqus_method_doc
    def getElemFaces(self) -> Tuple[MeshFace, ...]:
        """This method returns a tuple of element face objects that share the node.

        Returns
        -------
        faces: Sequence[MeshFace]
            A tuple of MeshFace objects
        """
        ...

    @abaqus_method_doc
    def getElements(self) -> Tuple[MeshElement, ...]:
        """This method returns a tuple of element objects that share the node.

        Returns
        -------
        elements: Sequence[MeshElement]
            A tuple of MeshElement objects
        """
        ...

    @abaqus_method_doc
    def getNodesByFeatureEdge(self, angle: float) -> MeshNodeArray:
        """This method returns an array of mesh node objects that are obtained by recursively
        finding adjacent nodes along a feature edge that are at an angle of less than or equal
        to the specified face angle.

        Parameters
        ----------
        angle
            A float specifying the value of the face angle in degrees.

        Returns
        -------
        nodes: MeshNodeArray
            A :py:class:`~abaqus.Mesh.MeshNodeArray.MeshNodeArray` object, which is a sequence of MeshNode objects
        """
        ...

    @abaqus_method_doc
    def setValues(self, label: Optional[int] = None) -> None:
        """This method modifies the MeshNode object.

        Parameters
        ----------
        label
            An Int specifying the node label. This member may only be edited if the node belongs to
            an orphan mesh part. The specified label must be non-negative and must not be in use by
            any other node of the same part.

        Returns
        -------
            None
        """
        ...



from typing import Optional, Tuple, Sequence, Union, List, Dict
from typing_extensions import Literal
from abaqus.XY.AreaStyle import AreaStyle
from abaqus.XY.LineStyle import LineStyle
from abaqus.XY.QuantityType import QuantityType
from abaqus.XY.SymbolStyle import SymbolStyle
from abaqus.XY.TextStyle import TextStyle
from abaqus.XY.XYData import XYData
from abaqus.XY.XYSession import XYSession
from abaqus.Odb.Odb import Odb
from abaqus.UtilityAndView.SymbolicConstant import abaqusConstants as C
from abaqus.UtilityAndView.abaqusConstants import (
    Boolean,
    OFF,
    ON,
    REAL,
)


from abaqus.UtilityAndView.SymbolicConstant import SymbolicConstant
from abaqus.PathAndProbe.Path import Path


def XYDataFromFile(
    fileName: str,
    name: str = "",
    sourceDescription: str = "",
    contentDescription: str = "",
    positionDescription: str = "",
    legendLabel: str = "",
    xValuesLabel: str = "",
    yValuesLabel: str = "",
    axis1QuantityType: Optional[QuantityType] = None,
    axis2QuantityType: Optional[QuantityType] = None,
    xField: int = 1,
    yField: int = 2,
    skipFrequency: Optional[int] = None,
) -> XYData:
    """This method creates an XYData object from data in an ASCII file.

    .. note:: 
        This function can be accessed by::

            session.XYDataFromFile
            xyPlot.XYDataFromFile

    Parameters
    ----------
    fileName
        A String specifying the name of the file from which the **X - Y** data will be read.
    name
        The repository key. If the name is not supplied, a default name in the form _temp#_ is
        generated and the XYData object is temporary.
    sourceDescription
        A String specifying the source of the **X - Y** data (e.g., “Entered from keyboard”, “Taken
        from ASCII file”, “Read from an ODB”, etc.). The default value is an empty string.
    contentDescription
        A String specifying the content of the **X - Y** data (e.g., “field 1 vs. field 2”). The
        default value is an empty string.
    positionDescription
        A String specifying additional information about the **X - Y** data (e.g., “for whole
        model”). The default value is an empty string.
    legendLabel
        A String specifying the label to be used in the legend. The default value is the name of
        the XYData object.
    xValuesLabel
        A String specifying the label for the X-values. This value may be overridden if the
        **X - Y** data are combined with other **X - Y** data. The default value is an empty string.
    yValuesLabel
        A String specifying the label for the Y-values. This value may be overridden if the
        **X - Y** data are combined with other **X - Y** data. The default value is an empty string.
    axis1QuantityType
        A :py:class:`~abaqus.XY.QuantityType.QuantityType` object specifying the QuantityType object associated to
        the X -axis1- values.
    axis2QuantityType
        A :py:class:`~abaqus.XY.QuantityType.QuantityType` object specifying the QuantityType object associated to
        the Y -axis2- values.
    xField
        An Int specifying the field from which the **X**-data will be read. Fields are delimited
        by spaces, tabs, or commas. The default value is 1.
    yField
        An Int specifying the field from which the **Y**-data will be read. Fields are delimited
        by spaces, tabs, or commas. The default value is 2.
    skipFrequency
        An Int specifying how often data rows will be skipped. A **skipFrequency** of 1 means skip
        every other row. The first row is always read. Possible values are **skipFrequency** ≥≥ 0.
        The default value is 0 (data are read from every row).

    Returns
    -------
    XYData
        An :py:class:`~abaqus.XY.XYData.XYData` object.

    Raises
    ------
    InvalidNameError
    RangeError
    """
    return XYData(())


def XYDataFromFreeBody(
    odb: Odb,
    force: Boolean = ON,
    moment: Boolean = OFF,
    heatFlowRate: Boolean = OFF,
    resultant: Boolean = ON,
    comp1: Boolean = OFF,
    comp2: Boolean = OFF,
    comp3: Boolean = OFF,
) -> List[XYData]:
    """This method creates a list of XYData objects by computing free body data from an Odb
    object.

    .. note:: 
        This function can be accessed by::

            session.XYDataFromFreeBody
            xyPlot.XYDataFromFreeBody

    Parameters
    ----------
    odb
        An :py:class:`~abaqus.Odb.Odb.Odb` object specifying the output database from which data will be read.
    force
        A boolean indicating whether to compute the force. The default is ON.
    moment
        A boolean indicating whether to compute the moment. The default is OFF.
    heatFlowRate
        A boolean indicating whether to compute the heat flow rate resultant magnitude. It is
        extracted only for viewcut based freebodies. The default is OFF.
    resultant
        A boolean indicating whether to compute the resultant. It applies only to **force** and
        **moment**. The default is ON.
    comp1
        A boolean indicating whether to compute the first component. It applies only to **force**
        and **moment**. The default is OFF.
    comp2
        A boolean indicating whether to compute the second component. It applies only to **force**
        and **moment**. The default is OFF.
    comp3
        A boolean indicating whether to compute the third component. It applies only to **force**
        and **moment**. The default is OFF.

    Returns
    -------
    List[XYData]
        A list of XYData objects.

    Raises
    ------
    InvalidNameError
    RangeError
    """
    return [XYData(())]


def XYDataFromHistory(
    odb: Odb,
    outputVariableName: str,
    steps: tuple,
    name: str = "",
    sourceDescription: str = "",
    contentDescription: str = "",
    positionDescription: str = "",
    legendLabel: str = "",
    skipFrequency: Optional[int] = None,
    numericForm: Literal[
        C.COMPLEX_MAGNITUDE, C.COMPLEX_PHASE, C.REAL, C.IMAGINARY, C.COMPLEX_VAL_AT_ANGLE
    ] = REAL,
    complexAngle: float = 0,
    stepTuple: Optional[int] = None,
) -> XYData:
    """This method creates an XYData object by reading history data from an Odb object.

    .. note:: 
        This function can be accessed by::

            session.XYDataFromHistory
            xyPlot.XYDataFromHistory

    Parameters
    ----------
    odb
        An :py:class:`~abaqus.Odb.Odb.Odb` object specifying the output database from which data will be read.
    outputVariableName
        A String specifying the output variable from which the **X - Y** data will be read.
    steps
        A sequence of Strings specifying the names of the steps from which data will be
        extracted.
    name
        The repository key. If the name is not supplied, a default name in the form _temp#_ is
        generated and the XYData object is temporary (this argument is required if the method is
        accessed from the session object).
    sourceDescription
        A String specifying the source of the **X - Y** data (for example, “Entered from keyboard”,
        “Taken from ASCII file”, “Read from an ODB”, etc.). The default value is an empty
        string.
    contentDescription
        A String specifying the content of the **X - Y** data (for example, “field 1 vs. field 2”).
        The default value is an empty string.
    positionDescription
        A String specifying additional information about the **X - Y** data (for example, “for whole
        model”). The default value is an empty string.
    legendLabel
        A String specifying the label to be used in the legend. The default value is the name of
        the XYData object.
    skipFrequency
        An Int specifying how often data frames will be skipped. If **skipFrequency** = 1, Abaqus
        will skip every other frame. The first frame is always read. Possible values are
        **skipFrequency** ≥≥ 0. The default value is 0 (data are read from every frame).
    numericForm
        A SymbolicConstant specifying the numeric form in which to display results that contain
        complex numbers. Possible values are COMPLEX_MAGNITUDE, COMPLEX_PHASE, REAL, IMAGINARY,
        and COMPLEX_VAL_AT_ANGLE. The default value is REAL.
    complexAngle
        A Float specifying the angle (in degrees) at which to display results that contain
        complex numbers when **numericForm** = COMPLEX_VAL_AT_ANGLE. The default value is 0.
    stepTuple
        A tuple of Integers specifying the steps to include when extracting data.

    Returns
    -------
    XYData
        An :py:class:`~abaqus.XY.XYData.XYData` object.

    Raises
    ------
    InvalidNameError
    RangeError
    """
    return XYData(())


def XYDataFromPath(
    path: Path,
    name: str,
    includeIntersections: Boolean,
    shape: Literal[C.UNDEFORMED, C.DEFORMED],
    pathStyle: Literal[C.PATH_POINTS, C.UNIFORM_SPACING],
    numIntervals: int,
    labelType: Literal[
        C.NORM_DISTANCE,
        C.SEQ_ID,
        C.TRUE_DISTANCE,
        C.TRUE_DISTANCE_X,
        C.TRUE_DISTANCE_Y,
        C.TRUE_DISTANCE_Z,
        C.X_COORDINATE,
        C.Y_COORDINATE,
        C.Z_COORDINATE,
    ],
    viewport: str = "",
    removeDuplicateXYPairs: Boolean = True,
    includeAllElements: Boolean = False,
    step: Optional[int] = None,
    frame: Optional[int] = None,
    variable: Optional[SymbolicConstant] = None,
    deformedMag: Optional[float] = None,
    numericForm: Literal[
        C.COMPLEX_MAGNITUDE, C.COMPLEX_PHASE, C.REAL, C.IMAGINARY, C.COMPLEX_VAL_AT_ANGLE
    ] = REAL,
    complexAngle: float = 0,
    projectOntoMesh: Boolean = False,
    projectionTolerance: float = 0,
) -> XYData:
    """This method creates an XYData object from path information.

    .. note:: 
        This function can be accessed by::

            session.XYDataFromPath
            xyPlot.XYDataFromPath

    Parameters
    ----------
    path
        A :py:class:`~abaqus.PathAndProbe.Path.Path` object to use in **X - Y** data generation.
    name
        A String specifying the repository key:for **session** 'name' is required argument and for
        **xyPlot** 'name' is optional argument.
    includeIntersections
        A Boolean specifying whether to include **X - Y** data for the intersections between the
        path and element faces or edges. The default value is False.
    shape
        A SymbolicConstant specifying the model shape to use. Possible values are UNDEFORMED and
        DEFORMED.
    pathStyle
        A SymbolicConstant specifying the path style. Possible values are PATH_POINTS and
        UNIFORM_SPACING.
    numIntervals
        An Int specifying the number of uniform-spacing intervals. The default value is 10.
    labelType
        A SymbolicConstant specifying the **X** label type to use. Possible values are
        NORM_DISTANCE, SEQ_ID, TRUE_DISTANCE, TRUE_DISTANCE_X, TRUE_DISTANCE_Y, TRUE_DISTANCE_Z,
        X_COORDINATE, Y_COORDINATE and Z_COORDINATE.
    viewport
        A String specifying the viewport name or an Int specifying the viewport id from which to
        obtain values. The default is the current viewport.
    step
        An Int identifying the step from which to obtain values. The default value is the
        current step.
    frame
        An Int identifying the frame from which to obtain values. The default value is the
        current frame.
    variable
        A tuple of tuples containing the descriptions of variables for which to extract data
        along the path. The default value is the current variable. Each tuple specifies the
        following:Variable label: A String specifying the variable; for example, 'U'.Variable
        output position: A SymbolicConstant specifying the output position. Possible values are
        ELEMENT_CENTROID, ELEMENT_FACE, ELEMENT_NODAL, GENERAL_PARTICLE, INTEGRATION_POINT,
        NODAL, WHOLE_ELEMENT, WHOLE_MODEL, WHOLE_PART_INSTANCE, and WHOLE_REGION.Refinement: A
        tuple specifying the refinement. If the refinement tuple is omitted, data are written
        for all components and invariants (if applicable). This element is required if the
        location dictionary (the following element in the tuple) is included. The refinement
        tuple contains the following:Type: A SymbolicConstant specifying the type of refinement.
        Possible values are INVARIANT and COMPONENT.Label: A String specifying the invariant or
        the component; for example, 'Mises' or 'S22'.Location: An optional Dictionary specifying
        the location. The dictionary contains pairs of the following:A String specifying the
        category selection label.A String specifying the section point label. For
        example::
        
            variable = ('S',INTEGRATION_POINT, ((COMPONENT, 'S22' ), ), ) 
            variable = (('S',INTEGRATION_POINT, ((COMPONENT, 'S11' ), ), ),
                        ('U',NODAL,((COMPONENT, 'U1'),)),) 
            variable = (('S', INTEGRATION_POINT, ((INVARIANT, 'Mises' ), ),
                        {'shell < STEEL > < 3 section points >':'SNEG,
                        (fraction = -1.0)', }), )
    deformedMag
        A tuple of three Floats specifying the deformation magnitude in the *X-*, *Y-*, and
        *Z-* planes. The default value is (1, 1, 1).
    numericForm
        A SymbolicConstant specifying the numeric form in which to display results that contain
        complex numbers. Possible values are COMPLEX_MAGNITUDE, COMPLEX_PHASE, REAL, IMAGINARY,
        and COMPLEX_VAL_AT_ANGLE. The default value is REAL.
    complexAngle
        A Float specifying the angle (in degrees) at which to display results that contain
        complex numbers when **numericForm** = COMPLEX_VAL_AT_ANGLE. The default value is 0.
    projectOntoMesh
        A Boolean to specify whether to consider the data points that do not lie on or inside
        the mesh. The default value is False.
    projectionTolerance
        A Float specifying the tolerance value for the projected distance considered for the
        data extraction when **projectOntoMesh** =  True. The default value is 0.

    Returns
    -------
    If **variable** specified has one fieldoutput: Returns an XYData object.
    If **variable** specified has more than one fieldoutputs: Returns list of XYData objects.

    Raises
    ------
    ErrorPathNotFound: Path not found
        If **path** is invalid.
    ErrorCurrentVPNotFound: Current viewport not found
        If **viewport** is invalid.
    ErrorInvalidUserStepAndFrame: The user step and frame specified have not been defined
        If **step** and/or **frame** are invalid.
    ErrorNoVarInPathExtract: No variable selection for XY data extraction from path
        If the **variable** argument is empty.
    ErrorUnavailableSelectedVariable: The selected variable is not available for the current frame
        If the specified output variable is not available in the output database.
    ErrorUnusableVarInPathExtract: Specified variable cannot be used in XY data extraction
        If the specified output variable cannot be used to obtain **X - Y** data from path.
    ErrorUnsupportedRefinementType: Unsupported refinement type
        If the SymbolicConstant specifying the refinement type is invalid.
    ErrorInvalidRefinementSpecification: Invalid refinement specification
        If the label specifying the refinement invariant or component is invalid.
    ErrorDeformedMagTupleInPathExtract: Deformed magnification tuple must contain X, Y and Z values
        If **deformedMag** does not contain three Floats.
    """
    return XYData(())


def XYDataFromShellThickness(
    odb: Odb,
    outputPosition: Literal[C.ELEMENT_CENTROID, C.ELEMENT_NODAL, C.INTEGRATION_POINT, C.NODAL],
    variable: Tuple[
        Tuple[
            str,
            Literal[
                C.ELEMENT_CENTROID,
                C.ELEMENT_FACE,
                C.ELEMENT_NODAL,
                C.GENERAL_PARTICLE,
                C.INTEGRATION_POINT,
                C.NODAL,
                C.WHOLE_ELEMENT,
                C.WHOLE_MODEL,
                C.WHOLE_PART_INSTANCE,
                C.WHOLE_REGION,
            ],
            Tuple[Literal[C.INVARIANT, C.COMPONENT], str],
            Dict[str, str],
        ],
        ...,
    ],
    elementSets: Union[str, Sequence[str]] = ...,
    elementLabels: Sequence[Tuple[str, Union[int, str]]] = ...,
    nodeSets: Union[str, Sequence[str]] = ...,
    nodeLabels: Sequence[Tuple[str, Union[int, str]]] = ...,
    numericForm: Literal[
        C.COMPLEX_MAGNITUDE, C.COMPLEX_PHASE, C.REAL, C.IMAGINARY, C.COMPLEX_VAL_AT_ANGLE
    ] = REAL,
    complexAngle: float = 0,
) -> List[XYData]:
    """This method creates a list of XYData objects by reading through the thickness field data
    from an Odb object.

    .. note:: 
        This function can be accessed by::

            session.XYDataFromShellThickness
            xyPlot.XYDataFromShellThickness

    Parameters
    ----------
    odb
        An :py:class:`~abaqus.Odb.Odb.Odb` object specifying the output database from which data will be read.
    outputPosition
        A SymbolicConstant specifying the position from which output will be read. Possible
        values are ELEMENT_CENTROID, ELEMENT_NODAL, INTEGRATION_POINT, and NODAL.
    variable
        A tuple of tuples containing the descriptions of variables for which to extract data from the
        field. Each tuple specifies the following:

        * Variable label: A String specifying the variable; for example, 'U'.
        * Variable output position: A SymbolicConstant specifying
            the output position. Possible values are ELEMENT_CENTROID, ELEMENT_FACE, ELEMENT_NODAL,
            GENERAL_PARTICLE, INTEGRATION_POINT, NODAL, WHOLE_ELEMENT, WHOLE_MODEL, WHOLE_PART_INSTANCE, and
            WHOLE_REGION.
        * Refinement: A tuple specifying the refinement. If
            the refinement tuple is omitted, data are written for all components and invariants (if
            applicable). This element is required if the location dictionary (the following element in the
            tuple) is included. The refinement tuple contains the following:

            * Type: A SymbolicConstant specifying the type of refinement. Possible values are INVARIANT and COMPONENT.
            * Label: A String specifying the invariant or the component; for example, 'Mises' or 'S22'.

        * Location: An optional Dictionary specifying the location. The
            dictionary contains pairs of the following:
            
            * A String specifying the category selection label.
            * A String specifying the section point label.

        For example::

            variable = ('S', INTEGRATION_POINT, ((COMPONENT, 'S22'), ), )
            variable = (('S', INTEGRATION_POINT, ((COMPONENT, 'S11'), ), ),
                        ('U', NODAL,((COMPONENT, 'U1'), )), )
            variable = (('S', INTEGRATION_POINT, ((INVARIANT, 'Mises'), ),
                            {'shell < STEEL > < 3 section points >': 'SNEG, (fraction = -1.0)', }), )
    elementSets
        A sequence of Strings specifying element sets or a String specifying a single element
        set.
    elementLabels
        A sequence of expressions specifying element labels per part instance in the model. Each
        part instance element expression is a sequence of a String specifying the part instance
        name and a sequence of element expressions; for example, 
        `(('partInstance1',(1,'7','3:15;3'),), ('partInstance2','8'),))`. The element
        expressions can be any of the following:An Int specifying a single element label; for
        example, `1`.A String specifying a single element label; for example, `'7'`.A String
        specifying a sequence of element labels; for example, `'3:5'` and `'3:15:3'`.
    nodeSets
        A sequence of Strings specifying node sets or a String specifying a single node set.
    nodeLabels
        A sequence of expressions specifying node labels per part instance in the model. Each
        part instance node expression is a sequence of a String specifying the part instance
        name and a sequence of node expressions; for example,
        `(('partInstance1',(1,'7','3:15;3'),), ('partInstance2','8'),))`. The node expressions
        can be any of the following:An Int specifying a single node label; for example, `1`.A
        String specifying a single node label; for example, `'7'`.A String specifying a sequence
        of node labels; for example, `'3:5'` and `'3:15:3'`.
    numericForm
        A SymbolicConstant specifying the numeric form in which to display results that contain
        complex numbers. Possible values are COMPLEX_MAGNITUDE, COMPLEX_PHASE, REAL, IMAGINARY,
        and COMPLEX_VAL_AT_ANGLE. The default value is REAL.
    complexAngle
        A Float specifying the angle (in degrees) at which to display results that contain
        complex numbers when **numericForm** = COMPLEX_VAL_AT_ANGLE. The default value is 0.

    Returns
    -------
    List[XYData]
        A list of XYData objects.

    Raises
    ------
    InvalidNameError
    RangeError
    """
    return [XYData(())]


def xyDataListFromField(
    odb: Odb,
    outputPosition: Literal[C.ELEMENT_CENTROID, C.ELEMENT_NODAL, C.INTEGRATION_POINT, C.NODAL],
    variable: Tuple[
        Tuple[
            str,
            Literal[
                C.ELEMENT_CENTROID,
                C.ELEMENT_FACE,
                C.ELEMENT_NODAL,
                C.GENERAL_PARTICLE,
                C.INTEGRATION_POINT,
                C.NODAL,
                C.WHOLE_ELEMENT,
                C.WHOLE_MODEL,
                C.WHOLE_PART_INSTANCE,
                C.WHOLE_REGION,
            ],
            Tuple[Literal[C.INVARIANT, C.COMPONENT], str],
            Dict[str, str],
        ],
        ...,
    ],
    elementSets: Union[Sequence[str], str] = ...,
    elementLabels: Sequence[Tuple[str, Union[int, str]]] = ...,
    nodeSets: Union[str, Sequence[str]] = ...,
    nodeLabels: Sequence[Tuple[str, Union[int, str]]] = ...,
    numericForm: Literal[
        C.COMPLEX_MAGNITUDE, C.COMPLEX_PHASE, C.REAL, C.IMAGINARY, C.COMPLEX_VAL_AT_ANGLE
    ] = REAL,
    complexAngle: float = 0,
    operator: Literal[
        C.ADD,
        C.SUBTRACT,
        C.MULTIPLY,
        C.DIVIDE,
        C.POWER,
        C.MINIMUM,
        C.MAXIMUM,
        C.AVERAGE,
        C.RANGE,
        C.SRSS,
        C.ABSOLUTE,
        C.UNARY_NEGATIVE,
        C.COSINE,
        C.HYPERBOLIC_COSINE,
        C.INVERSE_COSINE,
        C.SINE,
        C.HYPERBOLIC_SINE,
        C.INVERSE_SINE,
        C.TANGENT,
        C.HYPERBOLIC_TANGENT,
        C.INVERSE_TANGENT,
        C.EXPONENTIAL,
        C.NATURAL_LOG,
        C.LOG,
        C.SQUARE_ROOT,
        C.NORMALIZE,
        C.DEG2RAD,
        C.RAD2DEG,
        C.SMOOTH,
        C.SWAP,
        C.AVERAGE_ALL,
        C.MAXIMUM_ENVELOPE,
        C.MINIMUM_ENVELOPE,
        C.RANGE_ALL,
    ] = ...,
) -> List[XYData]:
    """This method creates a list of XYData objects by reading field data from an Odb object.

    .. note:: 
        This function can be accessed by::

            session.xyDataListFromField
            xyPlot.xyDataListFromField

    Parameters
    ----------
    odb
        An :py:class:`~abaqus.Odb.Odb.Odb` object specifying the output database from which data will be read.
    outputPosition
        A SymbolicConstant specifying the position from which output will be read. Possible
        values are ELEMENT_CENTROID, ELEMENT_NODAL, INTEGRATION_POINT, and NODAL.
    variable
        A tuple of tuples containing the descriptions of variables for which to extract data
        from the field. Each tuple specifies the following: 
        
        * Variable label: A String specifying the variable; for example, 'U'.
        * Variable output position: A SymbolicConstant specifying the output position. Possible values are
            ELEMENT_CENTROID, ELEMENT_FACE, ELEMENT_NODAL, GENERAL_PARTICLE, INTEGRATION_POINT, NODAL,
            WHOLE_ELEMENT,  WHOLE_MODEL, WHOLE_PART_INSTANCE, and WHOLE_REGION.
        * Refinement: A tuple specifying the refinement. If the refinement tuple is omitted, data are
            written  for all components and invariants (if applicable). This element is required if the
            location dictionary (the following element in the tuple) is included. The refinement tuple
            contains the following: 
            
            * Type: A SymbolicConstant specifying the type of refinement. Possible values are INVARIANT and
            COMPONENT.
            * Label: A String specifying the invariant or the component; for example, 'Mises' or 'S22'.
            
        * Location: An optional Dictionary specifying the location. The dictionary contains pairs of the
            following:
            
            * A String specifying the category selection label.
            * A String specifying the section point label.
            
        For example::

            variable = ('S', INTEGRATION_POINT, ((COMPONENT, 'S22'), ), )
            variable = (('S', INTEGRATION_POINT, ((COMPONENT, 'S11'), ), ),
                        ('U', NODAL,((COMPONENT, 'U1'), )), )
            variable = (('S', INTEGRATION_POINT, ((INVARIANT, 'Mises'), ),
                            {'shell < STEEL > < 3 section points >': 'SNEG, (fraction = -1.0)', }), )

    elementSets
        A sequence of Strings specifying element sets or a String specifying a single element
        set.
    elementLabels
        A sequence of expressions specifying element labels per part instance in the model. Each
        part instance element expression is a sequence of a String specifying the part instance
        name and a sequence of element expressions; for example,
        `(('partInstance1',(1,'7','3:15;3'),), ('partInstance2','8'),))`. The element
        expressions can be any of the following:

        * An Int specifying a single element label; for example, `1`.
        * A String specifying a single element label; for example, `'7'`.
        * A String specifying a sequence of element labels; for example, `'3:5'` and `'3:15:3'`.

    nodeSets
        A sequence of Strings specifying node sets or a String specifying a single node set.
    nodeLabels
        A sequence of expressions specifying node labels per part instance in the model. Each
        part instance node expression is a sequence of a String specifying the part instance
        name and a sequence of node expressions; for example,
        `(('partInstance1',(1,'7','3:15;3'),), ('partInstance2','8'),))`. The node expressions
        can be any of the following:

        * An Int specifying a single node label; for example, `1`.A
        * String specifying a single node label; for example, `'7'`.
        * A String specifying a sequence of node labels; for example, `'3:5'` and `'3:15:3'`.

    numericForm
        A SymbolicConstant specifying the numeric form in which to display results that contain
        complex numbers. Possible values are COMPLEX_MAGNITUDE, COMPLEX_PHASE, REAL, IMAGINARY,
        and COMPLEX_VAL_AT_ANGLE. The default value is REAL.
    complexAngle
        A Float specifying the angle (in degrees) at which to display results that contain
        complex numbers when **numericForm** = COMPLEX_VAL_AT_ANGLE. The default value is 0.
    operator
        A SymbolicConstant specifying the mathematical, trigonometric, logarithmic, exponential,
        or other operations. Possible values are ADD, SUBTRACT, MULTIPLY, DIVIDE, POWER,
        MINIMUM, MAXIMUM, AVERAGE, RANGE, SRSS, ABSOLUTE, UNARY_NEGATIVE, COSINE,
        HYPERBOLIC_COSINE, INVERSE_COSINE, SINE, HYPERBOLIC_SINE, INVERSE_SINE, TANGENT,
        HYPERBOLIC_TANGENT, INVERSE_TANGENT, EXPONENTIAL, NATURAL_LOG, LOG, SQUARE_ROOT,
        NORMALIZE, DEG2RAD, RAD2DEG, SMOOTH, SWAP, AVERAGE_ALL, MAXIMUM_ENVELOPE,
        MINIMUM_ENVELOPE, and RANGE_ALL. If no value is defined, no operation will be performed
        on the data, and the data will be saved as is.

    Returns
    -------
    List[XYData]
        A list of XYData objects.

    Raises
    ------
    InvalidNameError
    RangeError
    """
    return [XYData(())]



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .AnalyticalField import AnalyticalField


@abaqus_class_doc
class ExpressionField(AnalyticalField):
    """The ExpressionField object defines a spatially varying field whose value is calculated
    from a user-supplied mathematical expression.
    The ExpressionField object is derived from the AnalyticalField object.

    .. note::
        This object can be accessed by::

            import fields
            mdb.models[name].analyticalFields[name]
    """

    #: A String specifying the repository key.
    name: str

    #: A String specifying the Python expression to evaluate in space. Variables are X, Y, and
    #: Z; R, Th, and Z; or R, Th, and P based on the selected coordinate system.
    expression: str

    #: None or a DatumCsys object specifying the local coordinate system of the field. If
    #: **localCsys** = None, the field is defined in the global coordinate system. The default
    #: value is None.
    localCsys: Optional[str] = None

    #: A String specifying the description of the field. The default value is an empty string.
    description: str = ""

    @abaqus_method_doc
    def __init__(self, name: str, expression: str, localCsys: Optional[str] = None, description: str = ""):
        """This method creates an ExpressionField object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ExpressionField

        Parameters
        ----------
        name
            A String specifying the repository key.
        expression
            A String specifying the Python expression to evaluate in space. Variables are X, Y, and
            Z; R, Th, and Z; or R, Th, and P based on the selected coordinate system.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the field. If
            **localCsys** = None, the field is defined in the global coordinate system. The default
            value is None.
        description
            A String specifying the description of the field. The default value is an empty string.

        Returns
        -------
        ExpressionField
            An :py:class:`~abaqus.Field.ExpressionField.ExpressionField` object.

        Raises
        ------
        TextException
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(self, localCsys: Optional[str] = None, description: str = ""):
        """This method modifies the ExpressionField object.

        Parameters
        ----------
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the field. If
            **localCsys** = None, the field is defined in the global coordinate system. The default
            value is None.
        description
            A String specifying the description of the field. The default value is an empty string.
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from .BoundaryConditionState import BoundaryConditionState
from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class AcousticPressureBCState(BoundaryConditionState):
    """The AcousticPressureBCState object stores the propagating data for an acoustic pressure
    boundary condition in a step. One instance of this object is created internally by the
    AcousticPressureBC object for each step. The instance is also deleted internally by the
    AcousticPressureBC object.
    The AcousticPressureBCState object has no constructor or methods.
    The AcousticPressureBCState object is derived from the BoundaryConditionState object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].steps[name].boundaryConditionStates[name]

        The corresponding analysis keywords are:

        - BOUNDARY
    """

    #: A Float specifying the acoustic pressure magnitude.
    magnitude: Optional[float] = None

    #: A SymbolicConstant specifying the propagation state of the acoustic pressure magnitude.
    #: Possible values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    magnitudeState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the amplitude reference. Possible
    #: values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    amplitudeState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the BoundaryConditionState object. Possible values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - PROPAGATED_FROM_BASE_STATE
    #: - MODIFIED_FROM_BASE_STATE
    #: - DEACTIVATED_FROM_BASE_STATE
    #: - BUILT_INTO_MODES
    status: Optional[SymbolicConstant] = None

    #: A String specifying the name of the amplitude reference. The String is empty if the
    #: boundary condition has no amplitude reference.
    amplitude: str = ""



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from .InteractionState import InteractionState
from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class PressurePenetrationState(InteractionState):
    """The PressurePenetrationState object stores the propagating data of a PressurePenetration
    object in a step. One instance of this object is created internally by the
    PressurePenetration object for each step. The instance is also deleted internally by the
    PressurePenetration object.
    The PressurePenetrationState object has no constructor or methods.
    The PressurePenetrationState object is derived from the InteractionState object.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].steps[name].interactionStates[name]

        The corresponding analysis keywords are:

        - PRESSURE PENETRATION
    """

    #: A Float specifying the fraction of the current step time over which the fluid pressure
    #: on newly penetrated contact surface segments is ramped up to the current magnitude. The
    #: default value is 10-3.
    penetrationTime: Optional[float] = None

    #: A SymbolicConstant specifying the propagation state of the **penetrationTime** member.
    #: Possible values are UNSET, SET, UNCHANGED, and FREED.
    penetrationTimeState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the **amplitude** member. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    amplitudeState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the **criticalPressure** member.
    #: Possible values are UNSET, SET, UNCHANGED, and FREED.
    criticalPressureState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the **penetrationPressure** member.
    #: Possible values are UNSET, SET, UNCHANGED, and FREED.
    penetrationPressureState: Optional[SymbolicConstant] = None

    #: A tuple of Floats specifying the fluid pressure magnitude. For steady state dynamic
    #: analyses, a tuple of Complexes specifying the fluid pressure magnitude.
    penetrationPressure: Optional[float] = None

    #: A String specifying the name of the amplitude reference. The String is empty if the load
    #: has no amplitude reference.
    amplitude: str = ""

    #: A tuple of Floats specifying the critical contact pressure below which fluid penetration
    #: starts to occur.
    criticalPressure: Optional[float] = None

    #: A SymbolicConstant specifying the propagation state of the InteractionState object.
    #: Possible values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: Optional[SymbolicConstant] = None



from typing import Dict, Sequence

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from .EngineeringFeatureDisplayOptions import EngineeringFeatureDisplayOptions
from .GeometryDisplayOptions import GeometryDisplayOptions
from .MeshDisplayOptions import MeshDisplayOptions
from ..DisplayGroup.DisplayGroup import DisplayGroup
from ..DisplayGroup.DisplayGroupInstance import DisplayGroupInstance
from ..DisplayGroup.Leaf import Leaf
from ..UtilityAndView.abaqusConstants import Boolean, EMPTY_LEAF, OFF, SymbolicConstant, WIREFRAME
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class PartDisplayOptions:
    """The PartDisplayOptions object stores settings that specify how parts are to be displayed
    in a particular viewport. The PartDisplayOptions object has no constructor. When you
    create a new viewport, the settings are copied from the current viewport.

    .. note::
        This object can be accessed by::

            session.viewports[name].layers[name].partDisplay
            import part
            session.viewports[name].partDisplay
    """

    #: A Boolean specifying whether engineering features are shown. The default value is OFF.
    engineeringFeatures: Boolean = OFF

    #: A Boolean specifying whether to render the beam profiles. The default value is OFF.
    renderBeamProfiles: Boolean = OFF

    #: A Float specifying the beam profile scale factor. The beamScaleFactor must be greater
    #: than zero. The default value is 1.0.
    beamScaleFactor: float = 1

    #: A Boolean specifying whether the mesh should be displayed.
    mesh: Boolean = OFF

    #: A SymbolicConstant specifying how the image in the viewport is rendered. Possible values
    #: are WIREFRAME, HIDDEN, and SHADED. The default value is WIREFRAME.
    renderStyle: SymbolicConstant = WIREFRAME

    #: A :py:class:`~abaqus.DisplayGroup.DisplayGroup.DisplayGroup` object specifying the current display group and referring to an object in
    #: the **displayGroups** member of Session.
    displayGroup: DisplayGroup = DisplayGroup("dg", Leaf(EMPTY_LEAF))

    #: A repository of DisplayGroupInstance objects.
    displayGroupInstances: Dict[str, DisplayGroupInstance] = {}

    #: An :py:class:`~abaqus.DisplayOptions.EngineeringFeatureDisplayOptions.EngineeringFeatureDisplayOptions` object.
    engineeringFeatureOptions: EngineeringFeatureDisplayOptions = EngineeringFeatureDisplayOptions()

    #: A :py:class:`~abaqus.DisplayOptions.GeometryDisplayOptions.GeometryDisplayOptions` object.
    geometryOptions: GeometryDisplayOptions = GeometryDisplayOptions()

    #: A :py:class:`~abaqus.DisplayOptions.MeshDisplayOptions.MeshDisplayOptions` object.
    meshOptions: MeshDisplayOptions = MeshDisplayOptions()

    @abaqus_method_doc
    def setValues(
        self,
        renderStyle: Literal[C.SHADED, C.WIREFRAME, C.HIDDEN] = WIREFRAME,
        visibleDisplayGroups: Sequence[DisplayGroup] = (),
        engineeringFeatures: Boolean = OFF,
        renderBeamProfiles: Boolean = OFF,
        beamScaleFactor: float = 1,
    ):
        """This method modifies the PartDisplayOptions object.

        Parameters
        ----------
        renderStyle
            A SymbolicConstant specifying how the image in the viewport is rendered. Possible values
            are WIREFRAME, HIDDEN, and SHADED. The default value is WIREFRAME.
        visibleDisplayGroups
            A sequence of DisplayGroup objects specifying the DisplayGroups visible in the viewport.
            Currently the sequence can contain a maximum of one DisplayGroup object. The default
            value is an empty sequence.
        engineeringFeatures
            A Boolean specifying whether engineering features are shown. The default value is OFF.
        renderBeamProfiles
            A Boolean specifying whether to render the beam profiles. The default value is OFF.
        beamScaleFactor
            A Float specifying the beam profile scale factor. The beamScaleFactor must be greater
            than zero. The default value is 1.0.

        Raises
        ------
        RangeError
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class InelasticHeatFraction:
    """The InelasticHeatFraction object defines the fraction of the rate of inelastic
    dissipation that appears as a heat source.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].inelasticHeatFraction
            import odbMaterial
            session.odbs[name].materials[name].inelasticHeatFraction

        The corresponding analysis keywords are:

        - INELASTIC HEAT FRACTION
    """

    @abaqus_method_doc
    def __init__(self, fraction: float = 0):
        """This method creates an InelasticHeatFraction object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].InelasticHeatFraction
                session.odbs[name].materials[name].InelasticHeatFraction

        Parameters
        ----------
        fraction
            A Float specifying the fraction of inelastic dissipation rate that appears as a heat
            flux per unit volume. The fraction may include a unit conversion factor if required.
            Possible values are 0.0 ≤ **fraction** ≤ 1.0. The default value is 0.9.

        Returns
        -------
        InelasticHeatFraction
            An :py:class:`~abaqus.Material.Others.HeatTransfer.InelasticHeatFraction.InelasticHeatFraction` object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the InelasticHeatFraction object.

        Raises
        ------
        RangeError
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from ....UtilityAndView.abaqusConstants import Boolean, INCREMENTAL, MECHANICAL, OFF
from ....UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class UserMaterial:
    """The UserMaterial object defines material constants for use in subroutines UMAT, UMATHT,
    or VUMAT.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].userMaterial
            import odbMaterial
            session.odbs[name].materials[name].userMaterial

        The corresponding analysis keywords are:

        - USER MATERIAL
    """

    @abaqus_method_doc
    def __init__(
        self,
        type: Literal[C.THERMOMECHANICAL, C.THERMAL, C.MECHANICAL] = MECHANICAL,
        unsymm: Boolean = OFF,
        mechanicalConstants: tuple = (),
        thermalConstants: tuple = (),
        effmod: Boolean = OFF,
        hybridFormulation: Literal[C.UMAT, C.INCREMENTAL, C.TOTAL, C.INCOMPRESSIBLE] = INCREMENTAL,
    ):
        """This method creates a UserMaterial object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].UserMaterial
                session.odbs[name].materials[name].UserMaterial

        Parameters
        ----------
        type
            A SymbolicConstant specifying the type of material behavior defined by the command.
            Possible values are MECHANICAL, THERMAL, and THERMOMECHANICAL. The default value is
            MECHANICAL.
        unsymm
            A Boolean specifying if the material stiffness matrix, ∂Δσ/∂Δε, is not symmetric or,
            when a thermal constitutive model is used, if ∂f/∂(∂θ/∂x) is not symmetric. The default
            value is OFF. This argument is valid only for an Abaqus/Standard analysis.
        mechanicalConstants
            A sequence of Floats specifying the mechanical constants of the material. This argument
            is valid only when **type** = MECHANICAL or THERMOMECHANICAL. The default value is an empty
            sequence.
        thermalConstants
            A sequence of Floats specifying the thermal constants of the material. This argument is
            valid only when **type** = THERMAL or THERMOMECHANICAL. The default value is an empty
            sequence.
        effmod
            A Boolean specifying if effective bulk modulus and shear modulus are returned by user
            subroutine VUMAT. The default value is OFF. This argument is valid only in an
            Abaqus/Explicit analysis.
        hybridFormulation
            A SymbolicConstant to specify the formulation of the hybrid element with user subroutine
            UMAT. Possible values are TOTAL, INCREMENTAL, and INCOMPRESSIBLE. The default value is
            INCREMENTAL. This argument is valid only in an Abaqus/Standard analysis.

        Returns
        -------
        UserMaterial
            A :py:class:`~abaqus.Material.Others.User.UserMaterial.UserMaterial` object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the UserMaterial object.

        Raises
        ------
        RangeError
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class OdbSequenceAnalyticSurfaceSegment:
    """A sequence of AnalyticSurfaceSegment describing an analytic surface profile.

    .. note::
        This object can be accessed by::

            import odbAccess
            session.odbs[name].parts[name].analyticSurface.segments
            session.odbs[name].rootAssembly.instances[name].analyticSurface.segments
            session.odbs[name].steps[name].frames[i].fieldOutputs[name].values[i].instance.analyticSurface.segments
    """

    @abaqus_method_doc
    def __init__(self):
        """This method creates a OdbSequenceAnalyticSurfaceSegment object.

        .. note::
            This function can be accessed by::

                odbAccess.AnalyticSurfaceProfile

        Returns
        -------
        OdbSequenceAnalyticSurfaceSegment
            An :py:class:`~abaqus.Odb.OdbSequenceAnalyticSurfaceSegment.OdbSequenceAnalyticSurfaceSegment` object.
        """
        ...

    @abaqus_method_doc
    def Start(self, origin: tuple):
        """This method adds a AnalyticSurfaceSegment describing the first segment of the surface
        profile.

        Parameters
        ----------
        origin
            A sequence of Floats specifying the coordinates of start point.
        """
        ...

    @abaqus_method_doc
    def Line(self, endPoint: tuple):
        """This method adds a AnalyticSurfaceSegment describing the line segment of the surface
        profile.

        Parameters
        ----------
        endPoint
            A sequence of Floats specifying the coordinates of end point.
        """
        ...

    @abaqus_method_doc
    def Circle(self, center: tuple, endPoint: tuple):
        """This method adds a AnalyticSurfaceSegment describing a circular segment of the surface
        profile.

        Parameters
        ----------
        center
            A sequence of Floats specifying the coordinates of center of the circular segment.
        endPoint
            A sequence of Floats specifying the coordinates of end point of the circular segment.
        """
        ...

    @abaqus_method_doc
    def Parabola(self, middlePoint: tuple, endPoint: tuple):
        """This method adds a AnalyticSurfaceSegment describing a parabolic segment of the surface
        profile.

        Parameters
        ----------
        middlePoint
            A sequence of Floats specifying the coordinates of middle point of the parabolic
            segment.
        endPoint
            A sequence of Floats specifying the coordinates of end point of the parabolic segment.
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .GeometricRestriction import GeometricRestriction
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import Boolean, OFF


@abaqus_class_doc
class TopologyPointSymmetry(GeometricRestriction):
    """The TopologyPointSymmetry object defines a topology point symmetry geometric
    restriction.
    The TopologyPointSymmetry object is derived from the GeometricRestriction object.

    .. note::
        This object can be accessed by::

            import optimization
            mdb.models[name].optimizationTasks[name].geometricRestrictions[name]
    """

    #: A String specifying the geometric restriction repository key.
    name: str

    #: A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the geometric restriction is applied.
    #: When used with a TopologyTask, there is no default value. When used with a ShapeTask,
    #: the default value is MODEL.
    region: Region

    #: None or a DatumCsys object specifying the position of the symmetry point defined as the
    #: origin of a local coordinate system. If **csys** = None, the global coordinate system is
    #: used. When this member is queried, it returns an Int. The default value is None.
    csys: Optional[int] = None

    #: A Boolean specifying whether to ignore frozen areas. The default value is OFF.
    ignoreFrozenArea: Boolean = OFF

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        region: Region,
        csys: Optional[int] = None,
        ignoreFrozenArea: Boolean = OFF,
    ):
        """This method creates a TopologyPointSymmetry object.

        .. note::
            This function can be accessed by::

                mdb.models[name].optimizationTasks[name].TopologyPointSymmetry

        Parameters
        ----------
        name
            A String specifying the geometric restriction repository key.
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the geometric restriction is applied.
            When used with a TopologyTask, there is no default value. When used with a ShapeTask,
            the default value is MODEL.
        csys
            None or a DatumCsys object specifying the position of the symmetry point defined as the
            origin of a local coordinate system. If **csys** = None, the global coordinate system is
            used. When this member is queried, it returns an Int. The default value is None.
        ignoreFrozenArea
            A Boolean specifying whether to ignore frozen areas. The default value is OFF.

        Returns
        -------
        TopologyPointSymmetry
            A :py:class:`~abaqus.Optimization.TopologyPointSymmetry.TopologyPointSymmetry` object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(self, csys: Optional[int] = None, ignoreFrozenArea: Boolean = OFF):
        """This method modifies the TopologyPointSymmetry object.

        Parameters
        ----------
        csys
            None or a DatumCsys object specifying the position of the symmetry point defined as the
            origin of a local coordinate system. If **csys** = None, the global coordinate system is
            used. When this member is queried, it returns an Int. The default value is None.
        ignoreFrozenArea
            A Boolean specifying whether to ignore frozen areas. The default value is OFF.
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class Radiation:
    """The Radiation object specifies radiation for a contact interaction property.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].interactionProperties[name].radiation

        The corresponding analysis keywords are:

        - GAP RADIATION
    """

    #: A Float specifying the emissivity of the master surface.
    masterEmissivity: float

    #: A Float specifying the emissivity of the slave surface.
    slaveEmissivity: float

    #: A sequence of sequences of Floats specifying the following:Effective viewfactor, FF.Gap
    #: clearance, dd.
    table: tuple

    @abaqus_method_doc
    def __init__(self, masterEmissivity: float, slaveEmissivity: float, table: tuple):
        """This method creates a Radiation object.

        .. note::
            This function can be accessed by::

                mdb.models[name].interactionProperties[name].Radiation

        Parameters
        ----------
        masterEmissivity
            A Float specifying the emissivity of the master surface.
        slaveEmissivity
            A Float specifying the emissivity of the slave surface.
        table
            A sequence of sequences of Floats specifying the following:Effective viewfactor, FF.Gap
            clearance, dd.

        Returns
        -------
        Radiation
            A :py:class:`~abaqus.Interaction.Radiation.Radiation` object.
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the Radiation object."""
        ...



from typing import Union, Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from .Movie import Movie
from ..Session.SessionBase import SessionBase
from ..UtilityAndView.abaqusConstants import END_FRAME_TIME
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class AnimationSession(SessionBase):
    """The following commands operate on Session objects. For more information about the
    Session object, see Session object.

    .. note::
        This object can be accessed by::

            import animation
    """

    @abaqus_method_doc
    def writeImageAnimation(
        self, fileName: str, format: Literal[C.AVI, C.VRML, C.QUICKTIME, C.COMPRESSED_VRML], canvasObjects: tuple = ()
    ):
        """This method writes the animations present in the list of canvas objects to a file. It
        generates an animation file using the given file name and file format and uses the
        values in the appropriate options object.

        Parameters
        ----------
        fileName
            A String specifying the name of the animation file to generate.
        format
            A SymbolicConstant specifying the format of the generated file. Possible values are AVI,
            QUICKTIME, VRML, and COMPRESSED_VRML.
        canvasObjects
            A sequence specifying the canvas objects to capture. The default behavior is to capture
            all canvas objects.
        """
        # TODO: implement this method
        ...

    @abaqus_method_doc
    def Movie(
        self,
        name: str,
        fileName: str,
        startFrame: int = 0,
        endFrame: Optional[int] = None,
        timelineStartFrame: int = 0,
        timelineEndFrame: Optional[int] = None,
        timelineStartTime: float = 0,
        timelineEndTime: Union[Literal[C.END_FRAME_TIME], float] = END_FRAME_TIME,
    ) -> Movie:
        """This method creates a Movie object from the contents of the specified file.

        .. note::
            This function can be accessed by::

                session.Movie

        Parameters
        ----------
        name
            A String specifying the repository name for the movie.
        fileName
            A String specifying the file from which the movie is to be read. The file extension must
            be specified and indicates the movie format (.avi, .mov, .mpeg, or .wmv).
        startFrame
            An Int specifying the first frame to be displayed from this movie. The default value is
            0.
        endFrame
            An Int specifying the last frame to be displayed from this movie. A negative number will
            indicate reverse numbering: -1 is the last frame of the movie. The default value is −1.
        timelineStartFrame
            An Int specifying the global timeline frame number that corresponds to **startFrame**. A
            value of 0 will indicate the first frame to be displayed in the viewport. The default
            value is 0.
        timelineEndFrame
            An Int specifying the global timeline frame number that corresponds to **endFrame**. A
            negative number will indicate reverse numbering: -1 indicates the last frame to be
            displayed in the viewport. The default value is −1.
        timelineStartTime
            A Float specifying the global timeline time that corresponds to the time of
            **startFrame**. The default value is 0.0.
        timelineEndTime
            The SymbolicConstant END_FRAME_TIME or a Float specifying the global timeline time that
            corresponds to the time of **endFrame**. The SymbolicConstant END_FRAME_TIME indicates the
            time in this movie corresponding to **endFrame**. The default value is END_FRAME_TIME.

        Returns
        -------
        Movie
            A :py:class:`~abaqus.Animation.Movie.Movie` object.

        Raises
        ------
        ValueError: Unable to open movie file
            If **fileName** does not exist or can not be read.
        ValueError: Unsupported movie format.
            If **fileName** references an unsupported movie file format
        ValueError: Unable to decode movie file
            If the contents of **fileName** are corrupt or can not be decoded.
        """
        self.movies[name] = movie = Movie(
            name,
            fileName,
            startFrame,
            endFrame,
            timelineStartFrame,
            timelineEndFrame,
            timelineStartTime,
            timelineEndTime,
        )
        return movie



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from .BoundaryCondition import BoundaryCondition
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import SymbolicConstant, UNIFORM, UNSET
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class MagneticVectorPotentialBC(BoundaryCondition):
    """The MagneticVectorPotentialBC object stores the data for a magnetic vector potential
    boundary condition.
    The MagneticVectorPotentialBC object is derived from the BoundaryCondition object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].boundaryConditions[name]
    """

    #: A String specifying the boundary condition repository key.
    name: str = ""

    #: A SymbolicConstant specifying how the boundary condition is distributed spatially.
    #: Possible values are UNIFORM and USER_DEFINED. The default value is UNIFORM.
    distributionType: SymbolicConstant = UNIFORM

    #: A SymbolicConstant specifying the category of the boundary condition. Possible values
    #: are MECHANICAL and THERMAL.
    category: Optional[SymbolicConstant] = None

    #: A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the boundary condition is applied.
    region: Region = Region()

    #: None or a DatumCsys object specifying the local coordinate system of the boundary
    #: condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
    #: in the global coordinate system. The default value is None.
    localCsys: Optional[str] = None

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        createStepName: str,
        region: Region,
        component1: Optional[Literal[C.SET, C.UNSET]] = None,
        component2: Literal[C.SET, C.UNSET] = UNSET,
        component3: Literal[C.SET, C.UNSET] = UNSET,
        amplitude: str = UNSET,
        distributionType: Literal[C.USER_DEFINED, C.UNIFORM] = UNIFORM,
        localCsys: Optional[str] = None,
    ):
        """This method creates a MagneticVectorPotentialBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].MagneticVectorPotentialBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the boundary condition is applied.
        component1
            A Complex, or a SymbolicConstant specifying the magnetic vector potential component in
            the 1-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET
        component2
            A Complex, or a SymbolicConstant specifying the magnetic vector potential component in
            the 2-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        component3
            A Complex, or a SymbolicConstant specifying the magnetic vector potential component in
            the 3-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        distributionType
            A SymbolicConstant specifying how the boundary condition is distributed spatially.
            Possible values are UNIFORM and USER_DEFINED. The default value is UNIFORM.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        MagneticVectorPotentialBC
            A :py:class:`~abaqus.BoundaryCondition.MagneticVectorPotentialBC.MagneticVectorPotentialBC` object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        component1: Optional[Literal[C.SET, C.UNSET]] = None,
        component2: Literal[C.SET, C.UNSET] = UNSET,
        component3: Literal[C.SET, C.UNSET] = UNSET,
        amplitude: str = UNSET,
        distributionType: Literal[C.USER_DEFINED, C.UNIFORM] = UNIFORM,
        localCsys: Optional[str] = None,
    ):
        """This method modifies the data for an existing MagneticVectorPotentialBC object in the
        step where it is created.

        Parameters
        ----------
        component1
            A Complex, or a SymbolicConstant specifying the magnetic vector potential component in
            the 1-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET
        component2
            A Complex, or a SymbolicConstant specifying the magnetic vector potential component in
            the 2-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        component3
            A Complex, or a SymbolicConstant specifying the magnetic vector potential component in
            the 3-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        distributionType
            A SymbolicConstant specifying how the boundary condition is distributed spatially.
            Possible values are UNIFORM and USER_DEFINED. The default value is UNIFORM.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.
        """
        ...

    @abaqus_method_doc
    def setValuesInStep(
        self,
        stepName: str,
        component1: Optional[Literal[C.SET, C.UNCHANGED]] = None,
        component2: Optional[Literal[C.SET, C.UNCHANGED]] = None,
        component3: Optional[Literal[C.UNCHANGED]] = None,
        amplitude: str = "",
    ):
        """This method modifies the propagating data for an existing MagneticVectorPotentialBC
        object in the specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the boundary condition is modified.
        component1
            A Complex, or a SymbolicConstant specifying the magnetic vector potential component in
            the 1-direction. Possible values for the SymbolicConstant are SET and UNCHANGED.
        component2
            A Complex, or a SymbolicConstant specifying the magnetic vector potential component in
            the 2-direction. Possible values for the SymbolicConstant are SET and UNCHANGED.
        component3
            A Complex, or a SymbolicConstant specifying the magnetic vector potential component in
            the 3-direction. Possible values for the SymbolicConstant areSET and UNCHANGED.
        amplitude
            A String or a SymbolicConstant specifying the name of the amplitude reference. Possible
            values for the SymbolicConstant are UNCHANGED and FREED. UNCHANGED should be used if the
            amplitude is propagated from the previous analysis step. FREED should be used if the
            boundary condition is changed to have no amplitude reference. You should provide the
            **amplitude** argument only if it is valid for the specified step.
        """
        ...
